#include "scannerControl.h"

NodeManager nodes(&Serial, 3);
int status = STATUS_IDLE;

// WiFi settings
const char* ssid = "SSID";
const char* password = "password";

void init_web();

void homing() {
  // homing azimuth
  nodes.setEnabled(2, false);

  nodes.homing(0, -15000, 0, 1500.f);
  // homing LED brightness
  nodes.homing(2, 300, 255, 1500.f);

  nodes.setTarget(0, 100);
  nodes.setTarget(2, 70);
  nodes.runToTarget();
}

void init_nodes() {
  nodes.init();
  nodes.setMaxSpeed(0, 2300);
  nodes.setAcceleration(0, 2200);
  nodes.setMaxSpeed(1, 2000);
  nodes.setAcceleration(1, 1800);
  nodes.setMaxSpeed(2, 2500);
  nodes.setAcceleration(2, 6000);

  nodes.setEnabled(0, true);
  nodes.setEnabled(1, true);
  nodes.setEnabled(2, false);
}

void init_wifi() {
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.waitForConnectResult() != WL_CONNECTED) {
    // failed to connect. Rebooting...
    delay(1000);
    ESP.restart();
  }
}

void init_camera() {
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;

  #ifdef CAM_RAW
    config.xclk_freq_hz = 6000000;
    config.pixel_format = PIXFORMAT_YUV422;
    config.frame_size = FRAMESIZE_UXGA;
  #else
    config.xclk_freq_hz = 20000000;
    config.pixel_format = PIXFORMAT_JPEG;
    config.frame_size = FRAMESIZE_UXGA;
    config.jpeg_quality = 5;
  #endif
  
  config.fb_count = 1;

  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    return;
  }

  sensor_t * s = esp_camera_sensor_get();
  s->set_brightness(s, 0);     // -2 to 2
  s->set_contrast(s, 0);       // -2 to 2
  s->set_saturation(s, 0);     // -2 to 2
  s->set_special_effect(s, 0); // 0 to 6 (0 - No Effect, 1 - Negative, 2 - Grayscale, 3 - Red Tint, 4 - Green Tint, 5 - Blue Tint, 6 - Sepia)
  s->set_whitebal(s, 0);       // 0 = disable , 1 = enable
  s->set_awb_gain(s, 0);       // 0 = disable , 1 = enable
  s->set_wb_mode(s, 3);        // 0 to 4 - if awb_gain enabled (0 - Auto, 1 - Sunny, 2 - Cloudy, 3 - Office, 4 - Home)
  s->set_exposure_ctrl(s, 0);  // 0 = disable , 1 = enable
  s->set_aec2(s, 0);           // 0 = disable , 1 = enable
  s->set_ae_level(s, 0);       // -2 to 2
  s->set_aec_value(s, 1000);   // 0 to 1200
  s->set_gain_ctrl(s, 0);      // 0 = disable , 1 = enable
  s->set_agc_gain(s, 0);       // 0 to 30
  s->set_gainceiling(s, (gainceiling_t)0);  // 0 to 6
  s->set_bpc(s, 0);            // 0 = disable , 1 = enable
  s->set_wpc(s, 0);            // 0 = disable , 1 = enable
  s->set_raw_gma(s, 0);        // 0 = disable , 1 = enable
  s->set_lenc(s, 1);           // 0 = disable , 1 = enable
  s->set_hmirror(s, 0);        // 0 = disable , 1 = enable
  s->set_vflip(s, 0);          // 0 = disable , 1 = enable
  s->set_dcw(s, 0);            // 0 = disable , 1 = enable
  s->set_colorbar(s, 0);       // 0 = disable , 1 = enable
}

void setup() {
  #ifdef CAM_DEBUG
    Serial.begin(250000);
  #else
    init_nodes();
    esp_log_level_set("*", ESP_LOG_NONE);
  #endif

  init_wifi();
  init_web();
  init_camera();
  ArduinoOTA.begin();
  #ifdef CAM_DEBUG
    Serial.println("BOOT done.");
  #endif
}

void loop() {
  // put your main code here, to run repeatedly:
  ArduinoOTA.handle();
  delay(1);

  #ifdef CAM_DEBUG
    if (status != STATUS_IDLE) {
      status = STATUS_IDLE;
    }
  #else
    if (status == STATUS_HOMING) {
      homing();
      status = STATUS_IDLE;
    } else if (status == STATUS_MOVE) {
      nodes.runToTarget();
      status = STATUS_IDLE;
    }
  #endif
}
